/*********************************************
* vim: set sw=8 ts=8 si et :
* Author: Guido Socher, Copyright: GPL 
* This program is to test the led connected to LEDOUT (see below)
* 
* Clock frequency     : Internal clock 1 Mhz (factory default)
*********************************************/
#include <avr/io.h>
#define F_CPU 1000000UL  // 1 MHz
#include <util/delay.h>

#define LEDOUT PORTB1
#define LEDPORT PORTB
#define LEDDDR DDRB
#define LEDDDRPIN DDB1

unsigned int ledintensity=0; // 16 bit var and 10bit pwm

// _delay_ms uses a floating point datatype if you call
// that function in many places in your code then it becomes
// very fat. An integer is enough for us:
//
// delay x milliseconds:
void delay_ms(unsigned int xms)
{
        while(xms){
                _delay_ms(0.96);
                xms--;
        }
}

uint8_t getmode(void)
{
        // PD2= GND: blink, PD2= Vcc fading light
        if (bit_is_clear(PIND,PIND2)){
                return(1);
        }
        return(0);
}

// not used:
//void blink(void)
//{
//        /* led on, pin=0 */
//        LEDPORT &= ~(1<<LEDOUT);
//        delay_ms(500);
//        /* set output to 5V, LED off */
//        LEDPORT|= (1<<LEDOUT);
//        delay_ms(500);
//}

void init_ghostLed(void)
{
	// set up of Pulse Width Modulation (PWM)
	/* write high byte first for 16 bit register access: */
	TCNT1H=0;  /* set counter to zero*/
	TCNT1L=0;
	// COM1A1  COM1A0  
	//  1       0     Clear OC1A/OC1B on Compare Match (Set output to low level)
	//  1       1     Set OC1A/OC1B on Compare Match (Set output to high level)
	//
	// Fast PWM, Phase Correct, 10-bit (always count up, wrap counter at top)
	// See datasheet page 97 (settings) and 88 (description).
	TCCR1A=(1<<COM1A0)|(1<<COM1A1)|(1<<WGM10)|(1<<WGM11); 
	TCCR1B=(1<<CS10)|(1<<CS11)|(1<<WGM12); // 1/64 of clock speed

	// At what value to switch off the led (port OC1A=0 -> led=on)
	// ledintensity =0 -> led =off all the time
	// ledintensity=0x03FF -> led =on all the time
        // We we just switch off:
	OCR1AH=(ledintensity >> 8) & 0xff;
	OCR1AL=ledintensity & 0xff;

}

void ghostLed(void)
{
        int8_t step=2;
        unsigned int delay=0;
        uint8_t mode=0;
        uint8_t cnt=0;
	while (1) {
                if (mode){
                        // blink LED
                        cnt=0;
                        while(cnt<15){
                                cnt++;
                                if (ledintensity){
                                        delay=800; // a bit longer pauses between the short flashes
                                        ledintensity=0;
                                }else{
                                        delay=200; // short flashes
                                        ledintensity=0x03FF;
                                }
                                OCR1AH=(ledintensity >> 8) & 0xff;
                                OCR1AL=ledintensity & 0xff;
                                delay_ms(delay);
                        }
                        ledintensity=0;
                        delay=6000; // long off
                        OCR1AH=(ledintensity >> 8) & 0xff;
                        OCR1AL=ledintensity & 0xff;
                        delay_ms(delay);
                        delay_ms(delay);
                        mode=getmode();
                        continue;
                        // end blink LED
                }
                // fade LED
                //
                // --delay handling:
		delay=100;
                ledintensity+=step;
                if (ledintensity>0x02ff){
                        mode=getmode();
                        delay=30; //faster
                }
                if (ledintensity>0x00ff){
                        delay=50; //faster
                }
                //--
		if (ledintensity >0x3fc){
			step=-2; // down
			delay=1000; // wait at turnpoint
		}
		if (ledintensity <4){
                        mode=getmode();
			step=2; // up
			delay=1000; // wait at turnpoint
		}
		// At what value to switch off the led (port OC1A=0 -> led=on):
		// ledintensity =0 -> led =off, ledintensity=0x03FF -> led =on
		OCR1AH=(ledintensity >> 8) & 0xff;
		OCR1AL=ledintensity & 0xff;
		delay_ms(delay);
	}
}

int main(void)
{
        /* enable pin as output */
        LEDDDR|= (1<<LEDDDRPIN);
        // PD2 to check the mode:
        DDRD&= ~(1<<PIND2);
        PORTD|=1<<PIND2; // internal pullup resistor on
        init_ghostLed();

        ghostLed(); // does not return

        return(0);
}

